/***
 * $Source: /usr/home/ldl/langs/fm/RCS/fm.h,v $
 * $Revision: 0.19980909 $ $Date: 1998/09/09 20:46:27 $
 */
/*
 * FM - Femto-M[UMPS].
 * Data definitions and macros.
 */

/**
 *	Include some standard headers.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <setjmp.h>
/*	Go for minimal includes
#include <sys/types.h>
#include <stddef.h>
#include <signal.h>
#include <math.h>
#include <features.h>
#include <limits.h>
 */

/***
 *	Compilation control settings.
 */

/**
 *	Miscellaneous interpreter limits.
 */
#define	MAX_COMMANDS_PER_LINE	100	/* Obscene			*/
#define MAX_ROUTINENAME		8	/* Conformant			*/

/**
 *	Storage definitions.
 */
#ifndef	__BIT_TYPES_DEFINED__
#define	__BIT_TYPES_DEFINED__
typedef	__signed char		   int8_t;
typedef	unsigned char		 u_int8_t;
typedef	short			  int16_t;
typedef	unsigned short		u_int16_t;
typedef	int			  int32_t;
typedef	unsigned int		u_int32_t;
typedef	long long		  int64_t;
typedef	unsigned long long	u_int64_t;
#endif



/**
 *	If the compiler handles function prototypes
 */
#ifdef	PROTOTYPES
#define PROTO_ARGS(x)	x
#else /*PROTOTYPES*/
#define	PROTO_ARGS(x)	()
#endif/*PROTOTYPES*/

/**
 *	For defining "C" global variables
 */
#ifdef	IS_MAIN
#define	SC
#define IS(x)	x
#else /*IS_MAIN*/
#define SC	extern
#define IS(x)
#endif/*IS_MAIN*/

/***
 *	Some controlling manifest constants.
 */
#define	FUNCTIONS			/* enable functions */
#define	NID	16			/* # characters in id */


/***
 *	Manifest constants.
 */

/**
 *	Command argument flags.
 */
#define ARG_NONE	0x01		/*   not allowed		*/
#define ARG_OPT		0x02		/*   optional			*/
#define ARG_RQRD	0x04		/*   required			*/
#define ARG_LIST	0x08		/*   multiple, comma separated	*/


/***
 *	Structured types
 */

/**
 *	M Virtual Machine Opcode
 */
typedef					/* Type: MVMOp			*/
u_int8_t				/* ----------------------------	*/
	MVMOp;				/* M Virtual Machine Opcode	*/

/**
 *	The ubiquitious M string.
 *	Note: 'data' is NULL if 'size' is 0.
 */
typedef					/* Type: MStr			*/
struct {				/* ----------------------------	*/
	void   *data;			/* Address of first byte	*/
	size_t  size;			/* Length of string		*/
}	MStr;

typedef					/* Type: MStrFn			*/
MStr 					/* ----------------------------	*/
      (*MStrFn)();			/* Function* returning an MStr	*/

/**
 *	The structure of a precompiled source command:
 *		<op><postcond><arg>
 *	where:
 *		<op>       is an OP_xxx
 *		<postcond> is <cnt><string>
 *		<arg>      is <cnt><string>
 *	and
 *		<cnt>      is byte count for length of <string>
 *		<string>   is text of whatever was there
 */
typedef					/* Type: Command		*/
struct {				/* ----------------------------	*/
	MVMOp	cmd_op;			/* opcode.h		 OP_xxx	*/
	MStr	cmd_postcond;		/* Postconditional		*/
	MStr	cmd_arg;		/* Command argument		*/
}	Command;			/* ----------------------------	*/

/**
 *	The structure of a source code line
 */
typedef					/* Type: Line			*/
struct _line_ {				/* ----------------------------	*/
	struct _line_			/* Link to next line in		*/
	       *l_next;			/*   routine			*/
	char   *l_tag;			/* Line tag          (optional)	*/
	u_int8_t l_ncmd;		/* Number of commands in this	*/
	Command l_cmd[1];		/* Commands (dynamically alloc)	*/
}	Line;


/**
 *	Control stack entry
 */
/*typedef				/ * Type: Control		*/
struct control {			/* ----------------------------	*/
	struct	control	*c_fp;		/* Link */
	int	c_mode;			/* Control mode:	  C_xxx	*/
#define	C_TOP		0		/*   Top level			*/
#define	C_DLINE		1		/*   Do line			*/
#define	C_DGRP		2		/*   Do group			*/
#define	C_DALL		3		/*   Do all			*/
#define	C_FOR		4		/*   FOR loop			*/
	Line   *c_lp;			/* Saved current line		*/
	char   *c_tp;			/* Saved text pointer		*/
	struct sym			/*				*/
	       *c_sp;			/* Symbol (for)			*/
	double	c_limit;		/* Limit (for)			*/
	double	c_step;			/* Step (for)			*/
};


/**
 *	Symbol table entry...
 *
 *	NOTE: In the following implementation, each element of an array
 *	has a symbol table entry, where 'subs' is the subscript entry of
 *	the array.
 */
/*typedef				/ * Type: Symbol			*/
struct	sym {				/* ----------------------------	*/
	struct sym			/*				*/
	       *s_fp;			/* Link	to next frame		*/
	int	s_subs;			/* Subscript			*/
	int	s_type;			/* Type of entry	  S_xxx	*/
#define	S_SCAL		0		/*   Scalar			*/
#define	S_ARRAY		1		/*   An array			*/
#define	S_FUNC		2		/*   A function			*/
	union	{			/* Variant s_type of:		*/
		double	s_value;	/*   S_SCAL the data		*/
		double	(*s_fp)();	/*   S_FUNC function pointer	*/
	} s_un;				/*				*/
	char	s_id[1];		/* Name (dynamically allocated)	*/
};

/***
 *	External functions (in various files).
 */

Line   *allocLine	PROTO_ARGS((int));
Line   *compile		PROTO_ARGS((char *, int));
void	decompile	PROTO_ARGS((FILE *, Line *));
void	diag		PROTO_ARGS((char *));
void	do_zload	PROTO_ARGS((Command *));
void	do_zprint	PROTO_ARGS((Command *));
void	do_zremove	PROTO_ARGS((Command *));
void	do_zsave	PROTO_ARGS((Command *));
void	fatal		PROTO_ARGS((char *));
void	freeLine	PROTO_ARGS((Line *));
void	freeMStr	PROTO_ARGS((MStr));
char   *ifnc2name	PROTO_ARGS((MStrFn));
MStrFn	lkup_ifnc	PROTO_ARGS((char *, int *, int *));
MVMOp   lkup_opcode	PROTO_ARGS((char *, int *, int *));
MStrFn  lkup_svar	PROTO_ARGS((char *));
char   *op2name		PROTO_ARGS((MVMOp));
void	process		PROTO_ARGS((Line *));
MStr    str2MStr	PROTO_ARGS((char *));
char   *str2upper	PROTO_ARGS((char *, char *));
char   *svar2name	PROTO_ARGS((MStrFn));

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -	*/

/***
 *	C Global variables
 */

SC
char	cbuf[BUFSIZ];			/* Command buffer		*/
SC
Line   *cline		IS(= NULL);	/* Command Line compiled item	*/
SC
Line   *clp		IS(= NULL);	/* Current line pointer		*/
SC
Line   *routine		IS(= NULL);	/* Current routine		*/
SC
char    routinename[MAX_ROUTINENAME + 1] IS(= "");

/**
 *	Historical... delete or move above this line!
 */
SC
MStr	NULL_MStr	IS(= { 0 });	/* The "NULL" M String		*/
SC
int	intflag		IS(= 0);	/* Interrupt flag		*/
SC
int	mode;				/* Current processing mode	*/
SC
double	forlimit	IS(= 0.0);	/* Loop limit (for)		*/
SC
double	forstep		IS(= 0.0);	/* Loop step (for)		*/
SC
char   *ctp		IS(= NULL);	/* Current text pointer		*/
SC
struct sym
       *forsp		IS(= NULL);	/* Symbol pointer (for)		*/
SC
struct control
       *control		IS(= NULL);	/* Control stack		*/
SC
struct sym
       *sym		IS(= NULL);	/* Symbol table			*/

SC
jmp_buf	env;				/* Saved state for errors	*/


/* End of $Source: /usr/home/ldl/langs/fm/RCS/fm.h,v $ */
